@@ -12,7 +12,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * $Id: debug.c 411 2010-04-07 13:52:14Z ludovic.rousseau $
+ * $Id$
  */
 
 #include "debug.h"
@@ -37,7 +37,7 @@ void debug_print(int level, const char *file, int line, const char *format, ...)
   if (debug_level >= level) {
     /* is stdout is a tty */
     if (isatty(1)) {
-      const char *t = "\033[34mDEBUG"; /* blue */
+      const char *t = "\033[32mDEBUG"; /* green */
 
       if (-1 == level)
         t = "\033[31mERROR"; /* red */
