@@ -63,7 +63,7 @@ int pkcs11_pass_login(pkcs11_handle_t *h, int nullok)
 
   /* perform pkcs #11 login */
   rv = pkcs11_login(h, pin);
-  memset(pin, 0, strlen(pin));
+  cleanse(pin, strlen(pin));
   if (rv != 0) {
     set_error("pkcs11_login() failed: %s", get_error());
     return -1;
@@ -131,6 +131,43 @@ memcmp_pad_max(void *d1, size_t d1_len, void *d2, size_t d2_len,
 	return (0);
 }
 
+int get_random_value(unsigned char *data, int length)
+{
+  static const char *random_device = "/dev/urandom";
+  int rv, fh, l;
+
+  DBG2("reading %d random bytes from %s", length, random_device);
+  fh = open(random_device, O_RDONLY);
+  if (fh == -1) {
+    set_error("open() failed: %s", strerror(errno));
+    return -1;
+  }
+
+  l = 0;
+  while (l < length) {
+    rv = read(fh, data + l, length - l);
+    if (rv <= 0) {
+      close(fh);
+      set_error("read() failed: %s", strerror(errno));
+      return -1;
+    }
+    l += rv;
+  }
+  close(fh);
+  DBG5("random-value[%d] = [%02x:%02x:%02x:...:%02x]", length, data[0],
+      data[1], data[2], data[length - 1]);
+  return 0;
+}
+
+void cleanse(void *ptr, size_t len)
+{
+#ifdef HAVE_OPENSSL
+  OPENSSL_cleanse(ptr, len);
+#else
+  memset(ptr, 0, len);
+#endif
+}
+
 
 #ifdef HAVE_NSS
 /*
@@ -215,7 +252,7 @@ int crypto_init(cert_policy *policy) {
 }
 
 
-static SECMODModule *find_module_by_library(char *pkcs11_module)
+static SECMODModule *find_module_by_library(const char *pkcs11_module)
 {
   SECMODModule *module = NULL;
   SECMODModuleList *modList = SECMOD_GetDefaultModuleList();
@@ -225,7 +262,7 @@ static SECMODModule *find_module_by_library(char *pkcs11_module)
   DBG("Looking up module in list");
   for ( ; modList; modList = modList->next) {
     char *dllName = modList->module->dllName;
-    DBG2("modList = 0x%x next = 0x%x\n", modList, modList->next);
+    DBG2("modList = %p next = %p\n", modList, modList->next);
     DBG1("dllName= %s \n", dllName ? dllName : "<null>");
     if (dllName && strcmp(dllName,pkcs11_module) == 0) {
         module = SECMOD_ReferenceModule(modList->module);
@@ -266,7 +303,7 @@ int load_pkcs11_module(const char *pkcs11_module, pkcs11_handle_t **hp)
   /* specified module is not already loaded, load it now */
   moduleSpec = malloc(sizeof(SPEC_TEMPLATE) + strlen(pkcs11_module));
   if (!moduleSpec) {
-    DBG1("Malloc failed when allocating module spec", strerror(errno));
+    DBG1("Malloc failed when allocating module spec: %s", strerror(errno));
     free (h);
     return -1;
   }
@@ -609,7 +646,7 @@ void release_pkcs11_module(pkcs11_handle_t *h)
   if (h->module) {
     SECMOD_DestroyModule(h->module);
   }
-  memset(h, 0, sizeof(pkcs11_handle_t));
+  cleanse(h, sizeof(pkcs11_handle_t));
   free(h);
 
   /* if we initialized NSS, then we need to shut it down */
@@ -834,16 +871,6 @@ int sign_value(pkcs11_handle_t *h, cert_object_t *cert, CK_BYTE *data,
   return 0;
 }
 
-int get_random_value(unsigned char *data, int length)
-{
-  SECStatus rv = PK11_GenerateRandom(data,length);
-  if (rv != SECSuccess) {
-    DBG1("couldn't generate random number: %s", SECU_Strerror(PR_GetError()));
-  }
-  return (rv == SECSuccess) ? 0 : -1;
-}
-
-
 struct tuple_str {
     PRErrorCode	 errNum;
     const char * errString;
@@ -1081,7 +1108,6 @@ refresh_slots(pkcs11_handle_t *h)
   for (i = 0; i < h->slot_count; i++) {
     CK_SLOT_INFO sinfo;
     CK_TOKEN_INFO tinfo;
-    CK_RV rv;
 
     DBG1("slot %ld:", i + 1);
     rv = h->fl->C_GetSlotInfo(h->slots[i].id, &sinfo);
@@ -1119,7 +1145,6 @@ refresh_slots(pkcs11_handle_t *h)
 int init_pkcs11_module(pkcs11_handle_t *h,int flag)
 {
   int rv;
-  CK_ULONG i;
   /* CK_SLOT_ID_PTR slots; */
   CK_INFO info;
   /*
@@ -1181,7 +1206,7 @@ void release_pkcs11_module(pkcs11_handle_t *h)
   /* release all allocated memory */
   if (h->slots != NULL)
     free(h->slots);
-  memset(h, 0, sizeof(pkcs11_handle_t));
+  cleanse(h, sizeof(pkcs11_handle_t));
   free(h);
 }
 
@@ -1657,6 +1682,10 @@ int get_private_key(pkcs11_handle_t *h, cert_object_t *cert) {
     ,
     {CKA_ID, NULL, 0}
   };
+  CK_KEY_TYPE key_type;
+  CK_ATTRIBUTE attr_template[] = {
+    {CKA_KEY_TYPE, &key_type, sizeof(key_type)}
+  };
   CK_OBJECT_HANDLE object;
   CK_ULONG object_count;
   int rv;
@@ -1696,8 +1725,16 @@ int get_private_key(pkcs11_handle_t *h, cert_object_t *cert) {
     return -1;
   }
 
+  /* get private key type */
+  rv = h->fl->C_GetAttributeValue(h->session, object, attr_template, sizeof(attr_template) / sizeof(CK_ATTRIBUTE));
+  if (rv != CKR_OK) {
+    set_error("C_GetAttributeValue() failed! 0x%08lX", rv);
+    return -1;
+  }
+  DBG1("private key type: 0x%08lX", key_type);
+
   cert->private_key = object;
-  cert->key_type = CKK_RSA;
+  cert->key_type = key_type;
 
   return 0;
 
@@ -1723,8 +1760,14 @@ int sign_value(pkcs11_handle_t *h, cert_object_t *cert, CK_BYTE *data,
 	CK_ULONG length, CK_BYTE **signature, CK_ULONG *signature_length)
 {
   int rv;
+  int h_offset = 0;
+#ifdef USE_HASH_SHA1
   CK_BYTE hash[15 + SHA_DIGEST_LENGTH] =
       "\x30\x21\x30\x09\x06\x05\x2b\x0e\x03\x02\x1a\x05\x00\x04\x14";
+#else
+  CK_BYTE hash[19 + SHA256_DIGEST_LENGTH] =
+      "\x30\x31\x30\x0d\x06\x09\x60\x86\x48\x01\x65\x03\x04\x02\x01\x05\x00\x04\x20";
+#endif
   CK_MECHANISM mechanism = { 0, NULL, 0 };
 
 
@@ -1738,14 +1781,29 @@ int sign_value(pkcs11_handle_t *h, cert_object_t *cert, CK_BYTE *data,
     case CKK_RSA:
       mechanism.mechanism = CKM_RSA_PKCS;
       break;
+    case CKK_ECDSA:
+      mechanism.mechanism = CKM_ECDSA;
+#ifdef USE_HASH_SHA1
+      h_offset = 15;
+#else
+      h_offset = 19;
+#endif
+      break;
     default:
-      set_error("unsupported key type %d", cert->type);
+      set_error("unsupported private key type 0x%08X", cert->key_type);
       return -1;
   }
   /* compute hash-value */
+#ifdef USE_HASH_SHA1
+  DBG("hashing with SHA1");
   SHA1(data, length, &hash[15]);
   DBG5("hash[%ld] = [...:%02x:%02x:%02x:...:%02x]", sizeof(hash),
       hash[15], hash[16], hash[17], hash[sizeof(hash) - 1]);
+#else
+  SHA256(data, length, &hash[19]);
+  DBG5("hash[%ld] = [...:%02x:%02x:%02x:...:%02x]", sizeof(hash),
+      hash[19], hash[20], hash[21], hash[sizeof(hash) - 1]);
+#endif
   /* sign the token */
   rv = h->fl->C_SignInit(h->session, &mechanism, cert->private_key);
   if (rv != CKR_OK) {
@@ -1753,19 +1811,18 @@ int sign_value(pkcs11_handle_t *h, cert_object_t *cert, CK_BYTE *data,
     return -1;
   }
   *signature = NULL;
-  *signature_length = 128;
+  *signature_length = 64;
   while (*signature == NULL) {
     *signature = malloc(*signature_length);
     if (*signature == NULL) {
       set_error("not enough free memory available");
       return -1;
     }
-    rv = h->fl->C_Sign(h->session, hash, sizeof(hash), *signature, signature_length);
+    rv = h->fl->C_Sign(h->session, hash + h_offset, sizeof(hash) - h_offset, *signature, signature_length);
     if (rv == CKR_BUFFER_TOO_SMALL) {
       /* increase signature length as long as it it to short */
       free(*signature);
       *signature = NULL;
-      *signature_length *= 2;
       DBG1("increased signature buffer-length to %ld", *signature_length);
     } else if (rv != CKR_OK) {
       free(*signature);
@@ -1778,32 +1835,4 @@ int sign_value(pkcs11_handle_t *h, cert_object_t *cert, CK_BYTE *data,
       (*signature)[0], (*signature)[1], (*signature)[2], (*signature)[*signature_length - 1]);
   return 0;
 }
-
-int get_random_value(unsigned char *data, int length)
-{
-  static const char *random_device = "/dev/urandom";
-  int rv, fh, l;
-
-  DBG2("reading %d random bytes from %s", length, random_device);
-  fh = open(random_device, O_RDONLY);
-  if (fh == -1) {
-    set_error("open() failed: %s", strerror(errno));
-    return -1;
-  }
-
-  l = 0;
-  while (l < length) {
-    rv = read(fh, data + l, length - l);
-    if (rv <= 0) {
-      close(fh);
-      set_error("read() failed: %s", strerror(errno));
-      return -1;
-    }
-    l += rv;
-  }
-  close(fh);
-  DBG5("random-value[%d] = [%02x:%02x:%02x:...:%02x]", length, data[0],
-      data[1], data[2], data[length - 1]);
-  return 0;
-}
 #endif /* HAVE_NSS */
