@@ -17,7 +17,7 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  *
- * $Id: generic_mapper.c 445 2010-08-14 16:19:36Z ludovic.rousseau $
+ * $Id$
  */
 
 #define __GENERIC_MAPPER_C_
@@ -101,7 +101,7 @@ static char *generic_mapper_find_user(X509 *x509, void *context, int *match) {
 	/* and now return first nonzero item */
 	for (n=0;n<CERT_INFO_SIZE;n++) {
 	    char *str=entries[n];
-	    if (!str && !is_empty_str(str) ) {
+	    if (str && !is_empty_str(str) ) {
 		*match = 1;
 	    	return clone_str(str);
 	    }
@@ -131,7 +131,7 @@ static int generic_mapper_match_user(X509 *x509, const char *login, void *contex
 	/* and now try to match entries with provided login  */
 	for (n=0;n<CERT_INFO_SIZE;n++) {
 	    char *str=entries[n];
-	    if (str || is_empty_str(str) ) continue;
+	    if (!str || is_empty_str(str) ) continue;
 	    DBG2("Trying to match generic_mapped entry '%s' with login '%s'",str,login);
 	    if (ignorecase) {
 		if (! strcasecmp(str,login) ) return 1;
@@ -187,11 +187,12 @@ mapper_module * generic_mapper_module_init(scconf_block *blk,const char *name) {
 	else if (!strcasecmp(item,"email") )  id_type=CERT_EMAIL;
 	else if (!strcasecmp(item,"upn") )    id_type=CERT_UPN;
 	else if (!strcasecmp(item,"uid") )    id_type=CERT_UID;
+	else if (!strcasecmp(item,"serial") ) id_type=CERT_SERIAL;
 	else {
 	    DBG1("Invalid certificate item to search '%s'; using 'cn'",item);
 	}
 	pt = init_mapper_st(blk,name);
-	if (pt) DBG5("Generic mapper started. debug: %d, mapfile: '%s', ignorecase: %d usepwent: %d idType: '%s'",debug,mapfile,ignorecase,usepwent,id_type);
+	if (pt) DBG5("Generic mapper started. debug: %d, mapfile: '%s', ignorecase: %d usepwent: %d idType: '%d'",debug,mapfile,ignorecase,usepwent,id_type);
 	else DBG("Generic mapper initialization failed");
 	return pt;
 }
