@@ -6,6 +6,7 @@
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,10 +15,11 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+    02110-1301, USA.
 */
 
-/* $Id: card_eventmgr.c 484 2011-01-22 17:48:57Z ludovic.rousseau $ */
+/* $Id$ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -72,7 +74,7 @@ static void thats_all_folks(void) {
     /* We try to leave things as clean as possible */
     rv = SCardReleaseContext(hContext);
     if (rv != SCARD_S_SUCCESS) {
-        DBG1("SCardReleaseContext: %lX", rv);
+        DBG1("SCardReleaseContext: %X", rv);
     }
 
     /* free configuration context */
@@ -331,7 +333,7 @@ int main(int argc, char *argv[]) {
 	    pid_t pid = read_pidfile(pidfile);
 	    if (pid > 0)
 	    {
-		DBG1("Killing process: %ld", pid);
+		DBG1("Killing process: %ld", (long)pid);
 	    	kill(pid, SIGQUIT);
 	    }
 	    else
